# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Lockout',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('username', models.CharField(default=None, max_length=255, blank=True, null=True, verbose_name='username', db_index=True)),
                ('failed_attempts', models.PositiveIntegerField(default=0, verbose_name='failed attempts')),
                ('ip_address', models.IPAddressField(default=None, null=True, verbose_name='IP address', blank=True)),
                ('user_agent', models.CharField(default=None, max_length=1024, null=True, verbose_name='user agent', blank=True)),
                ('params_get', models.TextField(verbose_name='GET params')),
                ('params_post', models.TextField(verbose_name='POST params')),
                ('is_locked', models.BooleanField(default=False, db_index=True, verbose_name='locked')),
                ('is_expired', models.BooleanField(default=False, verbose_name='expired')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ('-created',),
                'db_table': 'cerberus_lockouts',
                'verbose_name': 'lockout',
                'verbose_name_plural': 'lockouts',
            },
        ),
    ]



from django.shortcuts import render

from cerberus.models import Lockout
from cerberus.settings import CERBERUS_MAX_ATTEMPTS, CERBERUS_LOCKOUT_TIME


def get_ip_address(request):
    # X_FORWARDED_FOR returns client1, proxy1, proxy2,...
    ip = request.META.get('HTTP_X_FORWARDED_FOR', None)
    return ip.split(', ')[0] if ip else request.META.get('REMOTE_ADDR', None)


def watch_logins(func):
    def new_func(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        ip = get_ip_address(request)

        # pass request.body as param
        #username = request.POST.get('username', None)
        lockout = get_lockout(ip)
        lockout = check_failed_login(request, response, lockout)

        if lockout.is_locked:
            response = get_locked_response(request, lockout)

        return response
    return new_func


def get_lockout(ip):
    """
    Returns the Lockout object for a given IP.
    """
    try:
        # get username to lookup
        lockout = Lockout.objects.filter(ip_address=ip, is_expired=False)[0]
    except IndexError:
        lockout = None

    if lockout:
        time_remaining = lockout.expiration_time
        if time_remaining is not None and time_remaining <= 0:
            # Unlock user
            lockout.is_expired = True
            lockout.save()
            return None

    return lockout


def check_failed_login(request, response, lockout):
    """
    If is a failed login, save the data in the database.
    It returns the Lockout instance.
    """
    ip = get_ip_address(request)
    user_agent = request.META.get('HTTP_USER_AGENT', None)
    username = request.POST.get('username', None)

    if not lockout:
        lockout = Lockout(ip_address=ip)

    if request.method == 'POST' and response.status_code != 302:
        # Failed login
        lockout.user_agent = user_agent
        lockout.username = username
        lockout.failed_attempts += 1
        lockout.params_get = request.GET
        lockout.params_post = request.POST

        if lockout.failed_attempts >= CERBERUS_MAX_ATTEMPTS:
            # Lock user
            lockout.is_locked = True
        lockout.save()
    elif request.method == 'POST' and response.status_code == 302 and lockout.id and not lockout.is_locked:
        # The user logged in successfully. Forgets about the failed login attempts
        lockout.is_expired = True
        lockout.save()

    return lockout


def get_locked_response(request, lockout):
    try:
        return render(
            request=request,
            template_name='cerberus/lockout.html',
            context={
                'lockout': lockout,
                'lockout_time': CERBERUS_LOCKOUT_TIME,
            }
        )
    except TypeError:
        return render(
            request=request,
            template_name='cerberus/lockout.html',
            context={
                'lockout': lockout,
                'lockout_time': CERBERUS_LOCKOUT_TIME
            }
        )



from django.db import models
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from cerberus.settings import CERBERUS_LOCKOUT_TIME


class Lockout(models.Model):
    username = models.CharField(max_length=255, verbose_name=_(u'username'), db_index=True,
        blank=True, null=True, default=None)
    failed_attempts = models.PositiveIntegerField(verbose_name=_(u'failed attempts'), default=0)
    ip_address = models.GenericIPAddressField(verbose_name=_(u'IP address'),
        blank=True, null=True, default=None)
    user_agent = models.CharField(max_length=1024, verbose_name=_(u'user agent'),
        blank=True, null=True, default=None)
    params_get = models.TextField(verbose_name=_(u'GET params'))
    params_post = models.TextField(verbose_name=_(u'POST params'))
    is_locked = models.BooleanField(verbose_name=_(u'locked'), default=False, db_index=True)
    is_expired = models.BooleanField(verbose_name=_(u'expired'), default=False)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = 'cerberus_lockouts'
        verbose_name = _(u'lockout')
        verbose_name_plural = _(u'lockouts')
        ordering = ('-created', )

    def __str__(self):
        return '%s: %s' % (self.username, self.ip_address)

    @property
    def expiration_time(self):
        """
        Returns the time until this access attempt is forgotten.
        """
        if CERBERUS_LOCKOUT_TIME <= 0:
            return None

        delta = now() - self.modified
        time_remaining = CERBERUS_LOCKOUT_TIME - delta.seconds
        return time_remaining

    def get_expiration_time_text(self):
        """
        Returns the text for the admin based on the time to forget
        """
        time_remaining = self.expiration_time

        if not self.is_locked:
            return _(u'Not locked yet')
        elif time_remaining is None:
            return _(u'Infinite')
        elif time_remaining <= 0:
            return _(u'Forgotten')
        else:
            return _(u'%(time_remaining)s seconds' % {'time_remaining': time_remaining})
    get_expiration_time_text.short_description = _(u'Expiration time')



from django.conf import settings

# Number of max failed login attempts
CERBERUS_MAX_ATTEMPTS = getattr(settings, 'CERBERUS_MAX_ATTEMPTS', 3)

# Number of seconds after the failed login attempts are forgotten in seconds
CERBERUS_LOCKOUT_TIME = getattr(settings, 'CERBERUS_LOCKOUT_TIME', 600)



from django.contrib import admin
from django.utils.translation import gettext as _

from cerberus.models import Lockout


@admin.register(Lockout)
class LockoutAdmin(admin.ModelAdmin):
    date_hierarchy = 'created'
    list_display = ['ip_address', 'username', 'user_agent', 'failed_attempts', 'get_expiration_time_text',
                    'is_locked', 'is_expired']
    list_filter = ['is_locked', 'is_expired']
    search_fields = ['ip_address', 'username', 'user_agent']
    fieldsets = (
        ('Main data', {
            'fields': ('ip_address', 'username', 'is_locked', 'is_expired', 'failed_attempts')
        }),
        ('Data recollected', {
            #'classes': ('collapse',),
            'fields': ('user_agent', 'params_get', 'params_post')
        }),
    )
    actions = ['lock', 'unlock']

    def lock(self, request, queryset):
        queryset.update(is_locked=True)
    lock.short_description = _(u'Lock users')

    def unlock(self, request, queryset):
        queryset.update(is_locked=False)
    unlock.short_description = _(u'Unlock users')
